.\"----------------------------------------------------------------------------
.\" Much of this manual was copied from the SIPP manual page which is:
.\" Copyright Jonas Yngvesson, Inge Wallin
.\" This program and documentation may be distributed freely under
.\" the terms of GNU GENERAL PUBLIC LICENSE.
.\" It is provided "as is" without express or implied warranty.
.\"----------------------------------------------------------------------------
.\" Parts of this manual that were not taken from the SIPP manual page are:
.\" Copyright 1992 Mark Diekhans
.\" Permission to use, copy, modify, and distribute this software and its
.\" documentation for any purpose and without fee is hereby granted, provided
.\" that the above copyright notice appear in all copies.  Mark Diekhans makes
.\" no representations about the suitability of this software for any purpose.
.\" It is provided "as is" without express or implied warranty.
.\"----------------------------------------------------------------------------
.\" $Id: tsipp.man,v 2.0 1992/11/02 03:55:46 markd Rel $
.\"----------------------------------------------------------------------------
.\"
.TH "tsipp" 1 "" "SIPP Tcl Interface"
.ad b
.SH NAME
tsipp - Tcl command interface to SIPP, the SImple Polygon Processor library.
'
.SH "INTRODUCTION"
.PP
Tcl-SIPP provides a Tcl command interface to SIPP, the SImple Polygon Processor
library.  This provides a command interface for creating 3-dimensional scenes
and rendering them using a scan-line z-buffer algorithm.  The Tcl interpretive
programming language is used to provide a powerful, yet simple interface to
this library.  The scene may be written to either a PPM format file, as
defined by the PBMPLUS toolkit or a RLE file, as defined by the Utah Raster
Toolkit.
'
.SH INVOCATION
.PP
Tcl-SIPP follows the calling conventions of the Tcl shell provided with
Extended Tcl.  The command line is:
.sp
.nf
.ft CW
    \fBtsipp\fR [-q] [[-f] \fIscript\fB]|[-c \fIcommand\fB] [\fIargs\fR]
.ft R
.fi
.sp
See the TclX manual page for details of the arguments and flags.
'
.SH "DATA TYPES AND RESOURCES"
'@help: sipp/intro/data-struct
'@brief: Data structures used by Tcl-SIPP.
.PP
The following data structures are used by Tcl-SIPP:
.IP
o \fIcolor\fB - A list containing the red, green and blue values for the
surface.  Each being a number between 0.0 and 1.0.
.IP
o \fIopacity\fB - Describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.
.IP
o \fIvertex\fB - A vertex is defined by a Tcl list of 3 floating point numbers
in the form: {x\ y\ z}.
.IP
o \fIvertex/texture pair\fB - A vertex and corresponding texture coordinates
are defined by a Tcl list of two list, each having 3 floating point numbers in
the form: {{x\ y\ z}\ {u\ v\ w}}.
.IP
o \fIvector\fB - A vector is defined by a Tcl list of 3 floating point numbers
in the form: {x\ y\ z}.
.IP
o \fIboolean\fB - Boolean arguments may be specified as `0' or `1', `true' or
`false', `t' or `f' (upper or lower case).  Only `0' or `1' are valid
in Tcl expressions and commands that return booleans only return `0' or `1'.
.IP
o \fIangle\fB - An angle may be expressed in degrees or radians.  If the
number is prefixed with an "R", then it is taken as radians, if it is prefixed
with "D", then it is taken as radians. (e.g. 1.2, R1.2, D10.5).  If no prefix
is specified, some commands assume degrees, some assume radians.  See the
documentation of the specific command.
.IP
o \fImatrix\fB - A 4x4 transformation matrix is represented as a list of 4
elements, each element being a row.  Each of the row elements is a list of 4
floating point numbers, representing the columns.  The values in column 4 must
be 0, 0, 0, 1.  For example:
.sp
.nf
.ft CW
        {{1.2  1.4  0.1  0}
         {2.1  2.3  1.6  0}
         {2.4  6.1  2.6  0}
         {2.9  1.1  3.6  1}}
.ft R
.fi
.PP
Be careful, many commands take lists of the above data types, which are 
themselves lists.
'
'@endhelp
'@help: sipp/intro/handles
'@brief: Handles used to reference sipp resources.
.PP
Various resources are created during the definition of a scene, these resources
are referenced in the Tcl code with handles.  A handle is similar to a file
id number returned by the \fBopen\fR system call.  It is an abstract entity
that provides a reference to a resource that is internally managed.   The
following handles for the following resources are provided:
.sp
.IP
o \fBcamera\fR - A camera which specifies a point of view.
The handle "\fBSTDCAMERA\fR" is predefined camera located at (0.0, 0.0 10.0),
looking at the origin, with the world y-axis as the up direction
and a focal factor of 0.25.  \fBSTDCAMERA\fR is an alias for a pre-created 
camera \fBcamera0\fR.  All standard camera commands will work on this camera,
including deleting it.
.IP
o \fBlight\fR - A light source or spot light.
.IP
o \fBsurface\fR - Surfaces that are constructed from polygons that have been
pushed on the polygon stack.
.IP
o \fBshader\fR - A shader defines a shading algorithm and surface attributes
required by the shader.  This handle can be used to define the way surfaces
will be shaded, including their color and attributes.
.IP
o \fBobject\fR - Objects defined in the SIPP internal database. A special
handle `\fBWORLD\fR' is used to specify the world object.
.IP
o \fBrlefile\fR - Utah Raster Toolkit RLE files.
.PP
When a delete operation is performed on a handle, a use count is decremented,
the resource is not actually release unless the use count goes to zero.  Thus
a handle to a object that has been added as a subobject of another object
may be deleted with out release the subobject.
'@endhelp
'
.SH "POLYGON AND SURFACE CREATION COMMANDS"
.PP
These commands provide for the definitions of polygons and the creation of
surfaces from the polygons.  There is a vertex stack for defining polygons and
a polygon stack for defining surfaces.  The surfaces are referenced by surface
handles.
'@help: sipp/surface/VertexPush
'@brief: Push a list of vertices onto the internal vertex stack. 
.TP
\fBSippVertexPush \fR[\fB-tex\fR] \fIvertexList\fR
.br
Push a list of vertices or vertex and texture coordinate pairs onto the
internal vertex stack.  There texture coordinates are currently not use in any
Tcl/SIPP rendering commands.  The vertices must be pushed on the vertex stack
\fIcounterclockwise\fR when looking at the "front" face of the polygon.
Otherwise the front of the surface will be defined in the wrong direction.  If
\fB-tex\fR is specified, then the list contains both vertices and texture
coordinates, each element being in the form "{x\ y\ z}\ {u\ v\ w}". 
If \fB-tex\fR is not specified, then the list contains just vertices, each
element being in the form "{x\ y\ z}".
'@endhelp
'
'@help: sipp/surface/PolygonPush
'@brief: Create a polygon and push it onto the polygon stack.
.TP
\fBSippPolygonPush \fR[[\fB-tex\fR] \fIvertexList\fR]
.br
Create a polygon from the vertices on the vertex stack and push it onto the
polygon stack. The vertex stack is empty and ready for a new polygon
definition after this operation.  Optional push a list of vertices or vertex
and texture coordinate pairs onto the internal vertex stack before creating
the polygon.  See \fBSippPolygonPush\fR for details.
.sp
If a vertex in the polygon is already defined in a previous polygon
that belongs to the same surface, the same vertex will be referenced,
i.e. vertices shared between polygons are only stored once, but they
must be repeated when defining the polygons.
'@endhelp
'
'@help: sipp/surface/SurfaceCreate
'@brief: Create a surface from the polygons on the polygon stack.
.TP
\fBSippSurfaceCreate \fIshaderhandle\fR
.br
Create a surface from the polygons on the polygon stack. A handle
to the newly-created surface is returned. The polygon stack is empty
afterwards. \fIshaderhandle\fR is a handle to the shader, which specifies
a shading algorithm and surface description.
See the section \fBSHADER COMMANDS\fR.
'@endhelp
'
'@help: sipp/surface/SurfaceDelete
'@brief: Decrement the reference count on surfaces.
.TP
\fBSippSurfaceDelete \fIsurfacelist\fR
.br
Decrement the reference count on all surface handles in \fIsurfacelist\fR.
If the reference count goes to zero the surface will be released.  In either
case, the surface handle will be released.
'@endhelp
'
'@help: sipp/surface/SurfaceSetShader
'@brief: Set the shader on specified surfaces.
.TP
\fBSippSurfaceSetShader \fIsurfacelist shaderhandle\fR
.br
Set the all of the surfaces in the list \fIsurfacelist\fR to be shaded with
the the shader \fIshaderhandle\fR.
'@endhelp
'
.SH "SHADER COMMANDS"
.P
These commands creates a surface shading description, which consists of a
shading algorithm specification and surface attributes.  They returns a 
shader handle that can be passed to other commands.
'
'@help: sipp/shaders/ShaderBasic
'@brief: Define a basic shader.
.TP
\fBSippShaderBasic \fIambient specular c3 color \fR[\fIopacity\fR]
.br
Create a basic shader, which is a modified and simplified version of Blinn's
shading model.
.IP
\fIambient\fR is a value between 0.0 and 1.0 which determines how much of the
base color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIspecular\fR is a value between 0.0 and 1.0 which is the fraction of
color specularly reflected.
.sp
\fIc3\fR is the shininess of the surface where 0.0 is the most shiny and 1.0
is the most dull.
.sp
\fIcolor\fR is a list containing the red, green and blue values for the
surface.  Each being a number between 0.0 and 1.0.
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderPhong
'@brief: Define a Phong shader.
.TP
\fBSippShaderPhong \fIambient diffuse specular spec_exp color \fR[\fIopacity\fR]
.br
Create a shader that implements the Phong illumination model.
.IP
\fIambient\fR is a value between 0.0 and 1.0 which determines how much of the
base color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIdiffuse\fR is a value between 0.0 and 1.0 specifying how much
light that is reflected diffusely from the surface.
.sp
\fIspecular\fR is a value between 0.0 and 1.0 which is the fraction of
color specularly reflected.
.sp
\fIspec_exp\fR is the exponent in the specular highlight calculation.
It specifies how "shiny" the surface is. Useful values are about 1 to
200, where 1 is a rather dull surface and 200 is a very shiny one.
.sp
\fIcolor\fR is a list containing the red, green and blue values for the
surface.  Each being a number between 0.0 and 1.0.
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderStrauss
'@brief: Define a Strauss intuitive shader.
.TP
\fBSippShaderStrauss \fIambient smoothness metalness color \fR[\fIopacity\fR]
.br
This is an implementation of a shader described by Paul
Strauss in IEEE CG&A Nov. 1990. In his article he explains that most shading
models in use today, ie Phong, Cook-Torrance, are difficult to use for
non-experts, and this for several reasons. The parameters and their effect on
a surface are non-intuitive and/or complicated. The shading model Strauss
designed has parameters that is easy to grasp and have a reasonably
deterministic effect on a surface, but yet produces very realistic results.
.IP
\fIambient\fR is a value between 0 and 1 which determines how much of the base
color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIsmoothness\fR is a value between 0 and 1 that describes how smooth the
surface is. This parameter controls both diffuse and specular reflections. 0
means a dull surface while 1 means a very smooth and shiny one.
.sp
\fImetalness\fR is also a value between 0 and 1. It describes how metallic the
material is. It controls among other things how much of the surface color
should be mixed into the specular reflections at different angles. 0 means a
non-metal while 1 means a very metallic surface.
.sp
\fIcolor\fR is a list containing the red, green and blue values for the
surface.  Each being a number between 0.0 and 1.0.
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderWood
'@brief: Define a simulated wood shader.
.TP
.nf
\fBSippShaderWood \fIambient specular c3 scale basecolor ringcolor \fR[\fIopacity\fR]
.fi
.br
This shader creates a simulated wood texture on a surface.
It uses two colors, one as the base (often lighter) color of the wood
and one as the color of the (often darker) rings in it.
The rings is put into the base color about the x-axis and are then distorted 
using \fInoise()\fR and \fIturbulence()\fR. A similar pattern is repeated at
regular intervals to create an illusion of logs or boards.
.IP
\fIambient\fR is a value between 0.0 and 1.0 which determines how much of the
base color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIspecular\fR is a value between 0.0 and 1.0 which is the fraction of
color specularly reflected.
.sp
\fIc3\fR is the shininess of the surface where 0.0 is the most shiny and 1.0
is the most dull.
.sp
\fIscale\fR is a factor which determines the size of the
wood pattern depending on the size of the texture coordinate system
in relation to the world coordinate system. You will have to
experiment some to get this right.
.sp
\fIbasecolor\fR is a list containing the red, green and blue values for the
wood base.  Each being a number between 0.0 and 1.0.
.sp
\fIringcolor\fR is a list containing the red, green and blue values for the
wood rings.
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderMarble
'@brief: Define a simulated marble shader.
.TP
.nf
\fBSippShaderMarble \fIambient specular c3 scale basecolor stripcolor \fR[\fIopacity\fR]
.fi
.br
This shader creates a simulated marble texture on a surface.
It uses two colors, one as the base material and one as the
interspersed material. The interspersed material is put into the
base material in strips that are distorted using \fInoise()\fR and
\fIturbulence()\fR. 
.IP
\fIambient\fR is a value between 0.0 and 1.0 which determines how much of the
base color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIspecular\fR is a value between 0.0 and 1.0 which is the fraction of
color specularly reflected.
.sp
\fIc3\fR is the shininess of the surface where 0.0 is the most shiny and 1.0
is the most dull.
.sp
\fIscale\fR is a factor which determines the size of the
marble pattern depending on the size of the texture coordinate system
in relation to the world coordinate system. 
.sp
\fIbasecolor\fR is a list containing the red, green and blue values for the
base material.  Each being a number between 0.0 and 1.0.
.sp
\fIstripcolor\fR is a list containing the red, green and blue values for the
interspersed material.
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderGranite
'@brief: Define a simulated granite shader.
.TP
.nf
\fBSippShaderGranite \fIambient specular c3 scale color1 color2 \fR[\fIopacity\fR]
.fi
.br
This shader is very similar to marble shader in that it also mixes two colors
using \fInoise()\fR and \fIturbulence()\fR. The difference is in how the
mixing is done. The two colors are mixed without treating them separately in
any way.
.IP
\fIambient\fR is a value between 0.0 and 1.0 which determines how much of the
base color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIspecular\fR is a value between 0.0 and 1.0 which is the fraction of
color specularly reflected.
.sp
\fIc3\fR is the shininess of the surface where 0.0 is the most shiny and 1.0
is the most dull.
.sp
\fIscale\fR is a factor which determines the size of the
granite pattern depending on the size of the texture coordinate system
in relation to the world coordinate system. 
.sp
\fIcolor1\fR and \fIcolor2\fR is a list containing the red, green and blue
values for the base material.  Each being a number between 0.0 and 1.0.
.sp
\fIstripcolor\fR is a list containing the red, green and blue values for the
interspersed material.
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderBozo
'@brief: Define a shader that uses noise() to chose colors.
.TP
.nf
\fBSippShaderBozo \fIcolorlist ambient specular c3 scale \fR[\fIopacity\fR]
.fi
.br
This shader uses \fInoise()\fR to chose a color from a fixed set.  The range
of possible return value from \fInoise()\fR are divided into parts of equal
size and each part is assigned a color. The size of the parts are dependent on
the number of colors.
.IP
\fIcolorlist\fR is a list contain one or more colors.  Each color is a list
containing the red, green and blue values for the surface.  Each being a
number between 0.0 and 1.0.
.sp
\fIambient\fR is a value between 0.0 and 1.0 which determines how much of the
base color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIspecular\fR is a value between 0.0 and 1.0 which is the fraction of
color specularly reflected.
.sp
\fIc3\fR is the shininess of the surface where 0.0 is the most shiny and 1.0
is the most dull.
.sp
\fIscale\fR is a factor which determines the size of the
pattern depending on the size of the texture coordinate system
in relation to the world coordinate system. 
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderBumpy
'@brief: Define a shader that adds bumps and holes to other shaders.
.TP
.nf
\fBSippShaderBumpy \fIshaderhandle scale\fR [\fBBUMPS\fR] [\fBHOLES\fR]
.fi
.br
This shader is a function that perturbates the normal of a
surface using \fInoise()\fR. Any other shader can be used to do the final
shading calculations.
.IP
\fIshaderhandle\fR is a handle to another shader and surface specification.
.sp
\fIscale\fR is a factor which determines the size of the
granite pattern depending on the size of the texture coordinate system
in relation to the world coordinate system. 
.sp
\fBBUMPS\fR indicates that only bumps standing out from the surface are
visible, the rest of the surface is smooth.
.sp
\fBHOLES\fR indicates that only bumps going into the surface are visible,
the rest of the surface is smooth.  If neither \fBHOLES\fR or \fBBUMPS\fR
or specified, then both are assumed to be on.
'@endhelp
'
'@help: sipp/shaders/ShaderPlanet
'@brief: Define a shader that creates a simulated planet surface.
.TP
.nf
\fBSippShaderPlanet \fIambient specular c3 \fR[\fIopacity\fR]
.fi
.br
This shader is a somewhat specialized shader that produces a texture
that resembles a planet surface. The planet is of the Tellus type with a
mixture of oceans and continents. Some of the surface is covered by
semi-transparent clouds which enhances the effect greatly. On the other hand,
no polar caps are provided and this decreases the realism.
.sp
The texture is 3-dimensional, so it is possible to create cube planets or even
planets with cut-out parts that still have surfaces that resemble the earth
surface. The texture is not scalable, and is designed to be used with texture
coordinates in the range -1.0 to 1.0, e.g. a unit sphere. Of course the world
coordinates need not have the same order of magnitude.
.IP
\fIambient\fR is a value between 0.0 and 1.0 which determines how much of the
base color of a surface that is visible when it is not illuminated by any
lightsource.
.sp
\fIspecular\fR is a value between 0.0 and 1.0 which is the fraction of
color specularly reflected.
.sp
\fIc3\fR is the shininess of the surface where 0.0 is the most shiny and 1.0
is the most dull.
.sp
\fIopacity\fR describes the opacity of a surface. 0.0 means totally
transparent, 1.0 means totally opaque.  May be a list of red, green and blue
opacity values or a single value to be used for all color channels.  Optional,
default is 1.0.
'@endhelp
'
'@help: sipp/shaders/ShaderDelete
'@brief: Delete a shader specification.
.TP
\fBSippShaderDelete \fIshaderlist\fR
.br
Delete the shader specifications in the list of shader hadles,
\fIshaderlist\fR.  This command only release the handles associated with the
shaders, the shader and surface descriptions still remain associated with any
surfaces that use them.
'@endhelp
'
.SH "OBJECT MANIPULATION COMMANDS"
.PP
These commands are provided for creating and manipulating objects.  An 
object is referenced by an object handle.
'@help: sipp/objects/ObjectCreate
'@brief: Create an empty object.
.TP
\fBSippObjectCreate\fR
.br
Create an empty object, i.e. an object with no surfaces or subobjects
in it. The transformation matrix in the new object will be a identity
matrix initially.  A handle to the object is returned which is passed to
the other object commands to specify this object. Object must be added as a
sub-object of the `\fBworld\fR' object or some other object to be become part
of the scene.
'@endhelp
'
'@help: sipp/objects/ObjectDelete
'@brief: Delete a list of objects.
.TP
\fBSippObjectDelete \fIobjectlist\fR
.br
Delete the all objects whose handles are in the list \fIobjectlist\fR. The
memory used by the objects and all its subobjects and surfaces is recursively
freed. \fIsipp\fR keeps track of internal references so objects and surfaces
referenced from somewhere else in \fIsipp\fR will not be deleted.
'@endhelp
'
'@help: sipp/objects/ObjectInstance
'@brief: Create a new instance of a previously defined object.
.TP
\fBSippObjectInstance \fIobjecthandle\fR
.br
Create a new instance of a previously defined object referenced by
\fIobjecthandle\fR. The lists of surfaces and subobjects in the object are not
copied, but a new reference with its own transformation matrix is created. The
matrix is set to the identity matrix. If the object is changed, i.e. if one of
its subobjects or surfaces are transformed, one is deleted or added, the
change will also be seen in the copy.
'@endhelp
'
'@help: sipp/objects/ObjectDup
'@brief: Copy recursively an object and its subobjects.
.TP
\fBSippObjectDup \fIobjecthandle\fR
.br
Copy recursively an object and its subobjects. The surfaces in the object tree
are not copied, only new references to them are made.
'@endhelp
'
'@help: sipp/objects/ObjectDeepDup
'@brief: Copy the entire tree of an object.
.TP
\fBSippObjectDeepDup \fIobjecthandle\fR
.br
Copy the entire tree of an object, including subobjects and all surfaces,
polygons and vertices. This is a costly operation if the object is complex.
'@endhelp
'
'@help: sipp/objects/ObjectAddSurface
'@brief: Add surfaces to an object.
.TP
\fBSippObjectAddSurface \fIobjecthandle surfacelist\fR
.br
Add the surfaces in the list \fIsurfacelist\fR to the specified object.
'@endhelp
'
'@help: sipp/objects/ObjectSubSurface
'@brief: Subtract surfaces from an object.
.TP
\fBSippObjectSubSurface \fIobjecthandle surfacelist\fR
.br
Remove (subtract) the surfaces in the list \fIsurfacelist\fR from the
specified object.
'@endhelp
'
'@help: sipp/objects/ObjectAddSubobj
'@brief: Add sub-objects to an specified object.
.TP
\fBSippObjectAddSubobj \fIobjecthandle subobjlist\fR
.br
Add the objects in the list \fIsubobjlist\fR to the specified object.
'@endhelp
'
'@help: sipp/objects/SubSubobj
'@brief: Subtract sub-objects from a object.
.TP
\fBSippObjectSubSubobj \fIobjecthandle subobjlist\fR
.br
Remove (subtract) the objects in the list \fIsubobjlist\fR from the specified
object.
'@endhelp
'
'@help: sipp/objects/ObjectGetTransf
'@brief: Return the transformation matrix for an object.
.TP
\fBSippObjectGetTransf \fIobjecthandle\fR
.br
Return the transformation matrix currently stored in the object specified 
object.
'@endhelp
'
'@help: sipp/objects/ObjectSetTransf
'@brief: Set the transformation matrix of an object.
.TP
\fBSippObjectSetTransf \fIobject matrix\fR
.br
Set the transformation matrix of the specified object to \fImatrix\fR.
'@endhelp
'
'@help: sipp/objects/ObjectClearTransf
'@brief: Set the transformation matrix of an object to the unit matrix.
.TP
\fBSippObjectClearTransf \fIobject\fR
.br
Set the transformation matrix of the specified object to the unit matrix.
'@endhelp
'
'@help: sipp/objects/ObjectTransform
'@brief: Post multiply a matrix into the transformation matrix of an object.
.TP
\fBSippObjectTransform \fIobject matrix\fR
.br
Post multiply specified matrix into the transformation matrix of \fIobject\fR.
'@endhelp
'
'@help: sipp/objects/ObjectRotateX
'@brief: Rotate an object about the X axis.
.TP
\fBSippObjectRotateX \fIobject angle\fR
.br
Rotate the \fIobject\fR \fIangle\fR radians or degrees about the X axis. The
angle is radians if no prefix or if the number is prefixed with an "R" and is
degrees, prefixed with "D".
'@endhelp
'
'@help: sipp/objects/ObjectRotateY
'@brief: Rotate an object about the Y axis.
.TP
\fBSippObjectRotateY \fIobject angle\fR
.br
Rotate the \fIobject\fR \fIangle\fR radians or degrees about the Y axis. The
angle is radians if no prefix or if the number is prefixed with an "R" and is
degrees, prefixed with "D".
'@endhelp
'
'@help: sipp/objects/ObjectRotateZ
'@brief: Rotate an object about the Z axis.
.TP
\fBSippObjectRotateZ \fIobject angle\fR
.br
Rotate the \fIobject\fR \fIangle\fR radians or degrees about the Z axis. The
angle is radians if no prefix or if the number is prefixed with an "R" and is
degrees, prefixed with "D".
'@endhelp
'
'@help: sipp/objects/ObjectRotate
'@brief: Rotate an object about an arbitrary line.
.TP
\fBSippObjectRotate \fIobjecthandle point vector angle\fR
.br
Rotate the specified object \fIangle\fR radians or degrees about the 
line given by \fIpoint\fR and \fIvector\fR starting in
that point.
The angle is radians if no prefix or if the number is prefixed with an "R"
and is degrees, prefixed with "D".
'@endhelp
'
'@help: sipp/objects/ObjectScale
'@brief: Scale an object.
.TP
\fBSippObjectScale \fIobjecthandle factor\fR|{\fIxfactor yfactor zfactor\fR}
.br
Scale the specified object with the scaling factors \fIxfactor,\
yfactor\fR and \fIzfactor\fR in the main directions respectively.  If only 
\fIfactor\fR is specified, it is used as the scaling factors in all directions.
'@endhelp
'
'@help: sipp/objects/ObjectMove
'@brief: Move (translate) an object.
.TP
\fBSippObjectMove \fIobjecthandle {\fIxdist ydist zdist\fR}
.br
Move (translate) the specified  \fIxdist, \fIydist\fR and \fIzdist\fR in the
three main directions, respectively.
'@endhelp
'
.SH "PRIMITIVE OBJECT CREATION COMMANDS"
.PP
These commands create primitive objects of specific shapes.
'
'
'@help: sipp/primitves/Texture-Map
'@brief: Texture mapping algorithms for primitives.
.TP
Texture mappings algorithms are supported for primitive objects:
.sp
Each object primitive which can be created in SIPP has an argument
that describes what kind of texture coordinates should be assigned to
the surface of the object. This parameter can have one of the following
predefined values:
'
.IP
o \fBNATURAL\fR - This value tell SIPP to use a two dimensional mapping which
is "natural" for this particular object. It might be one of the other
available mappings or it might be something unique for the object.  The
description of the functions for creating the individual objects specifies how
this mapping is done.
'
.IP
o \fBCYLINDRICAL\fR - A two dimensional mapping. The coordinates are assigned
as if the object were projected on a cylinder surrounding the object and
centered on the z-axis object. The coordinates are mapped so that `x' goes
from 0 to 1 around the base of the cylinder and `y' goes from 0 to 1 from
bottom to top on it.
'
.IP
o \fBSPHERICAL\fR - Same as `CYLINDRICAL', but the object are projected on a
sphere surrounding it instead.
'
.IP
o \fBWORLD\fR - A three dimensional mapping. The texture coordinates are the
same three dimensional coordinates as the world coordinates of the object at
creation time.
'@endhelp
'
'@help: sipp/primitves/Torus
'@brief: Create a torus object.
.TP
.nf
\fBSippTorus \fIbigradius smallradius radialres tuberes shaderhandle\fR [\fItexture\fR]
.fi
.br
Create a torus centered about the origin and with the z-axis pointing up
through the ring.  The surface is shaded with the algorithm and surface
parameters specified by \fIshaderhandle\fR.  \fIbigradius\fR is the radius of
the ring and \fIsmallradius\fR is the radius of the "tube" itself.
\fIradialres\fR is the number of polygons that will be created radially around
the ring and \fItuberes\fR is the number of polygons that will be created
around the tube.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR texture mapping is a two dimensional mapping with the
\fIx\fR coordinate going around the \fIsmall\fR circle and the \fIy\fR
coordinate going around the \fIlarge\fR circle.
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
.br
'@endhelp
'
'@help: sipp/primitves/Cone
'@brief: Create a cone object.
.TP
.nf
\fBSippCone \fIbottomradius topradius length resolution shaderhandle\fR [\fItexture\fR]
.fi
.br
Create a cone centered about the origin and with the z-axis along the cones
main axis.  The surface is shaded with the algorithm and surface parameters
specified by \fIshaderhandle\fR.  \fIbottomradius\fR and \fItopradius\fR
determines the radius of the cone in its bottom and top, if both of these are
non zero a truncated cone is created. \fIlength\fR defines the length of the
cone.  \fIresolution\fR is the number of polygons that will be created
radially around the rim.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR' texture mapping is \fBCYLINDRICAL\fR.
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
'@help: sipp/primitves/Cylinder
'@brief: Create a cylinder object.
.TP
\fBSippCylinder \fIradius length resolution shaderhandle\fR [\fItexture\fR]
.br
Create a cylinder centered about the origin and with the z-axis along the
cylinders main axis.  The surface is shaded with the algorithm and surface
parameters specified by \fIshaderhandle\fR.  \fIradius\fR and \fIlength\fR
defines the size of the cylinder. \fIresolution\fR is the number of polygons
that will be created radially around the rim.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR' texture mapping is \fBCYLINDRICAL\fR.
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
'@help: sipp/primitves/Ellipsoid
'@brief: Create a ellipsoid object.
.TP
.nf
\fBSippEllipsoid {\fIxradius yradius zradius\fR} \fIresolution\fR \fIshaderhandle\fR [\fItexture\fR]
.fi
.br
Create a ellipsoid centered about the origin.
The surface is shaded with the algorithm and surface parameters specified
by \fIshaderhandle\fR.
\fIxradius\fR, \fIyradius\fR and \fIzradius\fR
defines the size of the ellipsoid. \fIresolution\fR
is the number of polygons that will be created around it's "equator".
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR texture mapping is \fBSPHERICAL\fR.
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
'@help: sipp/primitves/Sphere
'@brief: Create a sphere object.
.TP
\fBSippSphere \fIradius resolution shaderhandle\fR [\fItexture\fR]
.br
Create a sphere centered about the origin.
The surface is shaded with the algorithm and surface parameters specified
by \fIshaderhandle\fR.
\fIradius\fR defines the size of the sphere. \fIresolution\fR
is the number of polygons that will be created around it's "equator".
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR texture mapping is \fBSPHERICAL\fR.
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
'@help: sipp/primitves/Prism
'@brief: Create a prism object.
.TP
\fBSippPrism \fI2dpointlist length shaderhandle\fR [\fItexture\fR]
.br
Creates a prism defined by the polygon in \fI2dpointlist\fR.
The surface is shaded with the algorithm and surface parameters specified
by \fIshaderhandle\fR.
\fI2dpointlist\fR is a list of lists of x and y pairs.
The prism will
ascend and descend equally far (\fIlength\fR / 2.0) from the x-y plane
along the z axis.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR texture mapping is similar to \fBCYLINDRICAL\fR but the
\fIx\fR coordinate is not taken from projection on a cylinder but is evenly
distributed around the perimeter. An odd thing in all the 2D mappings
(all except \fBWORLD\fR) for the prism is that the top face will have
texture coordinates (0.0, 1.0) while the bottom will get (0.0, 0.0).
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
'@help: sipp/primitves/Block
'@brief: Create a block object.
.TP
.nf
\fBSippBlock {\fIxsize ysize zsize\fR} \fIshaderhandle\fR [\fItexture\fR]
.fi
.br
Creates a square block with the size defined by \fIxsize\fR, \fIzsize\fR and
\fIzsize\fR.  The surface is shaded with the algorithm and surface parameters
specified by \fIshaderhandle\fR.  The block is centered about the origin.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR texture mapping is similar to \fBCYLINDRICAL\fR but the
\fIx\fR coordinate is not taken from projection on a cylinder but is evenly
distributed around the perimeter. An odd thing in all the 2D mappings
(all except \fBWORLD\fR) for the block is that the top face will have
texture coordinates (0.0, 1.0) while the bottom will get (0.0, 0.0).
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
'@help: sipp/primitves/Cube
'@brief: Create a cube object.
.TP
.nf
\fBSippCube \fIsize shaderhandle\fR [\fItexture\fR]
.fi
.br
Create a cube with the side of length \fIside\fR.  The surface is shaded with
the algorithm and surface parameters specified by \fIshaderhandle\fR.  The
cube is centered about the origin. \fIsurface\fR is the surface description
used by \fIshader()\fR which is the shading function used when shading the
cube.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The \fBNATURAL\fR texture mapping is similar to \fBCYLINDRICAL\fR but the
`x' coordinate is not taken from projection on a cylinder but is evenly
distributed around the perimeter. An odd thing in all the 2D mappings
(all except \fBWORLD\fR) for the cube is that the top face will have
texture coordinates (0.0, 1.0) while the bottom will get (0.0, 0.0).
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
'@help: sipp/primitves/BezierCurve
'@brief: Define an object using a Bezier curve rotated about the z-axis.
.TP
.nf
\fBSippBezierCurve \fIresolution vertexlist curvelist shaderhandle\fR [\fItexture\fR]
.fi
Define an object using a Bezier curve rotated about the z-axis.
\fIResolution\fR it the number of polygons to tessellate the surface into.
\fIVertexlist\fR is a list of vertices of the curve.  Each element of the list
is a list contain the X, Y and Z vertices.
.sp
\fICurvelist\fR a list of indices into \fIvertexlist\fR that defined the curve.
Each element of the list contains a list of 4 indices. These indices are zero
based. For example:
.sp
.nf
.ft CW
    {{0 1 2 3}
     {4 4 5 7}
     {6 7 8 9}}
.ft R
.fi
.sp
\fIshaderhandle\fRis handle to a previously created surface shading
specification.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The texture coordinates are a bit special for these surfaces.
\fBSPHERICAL\fR and \fBCYLINDRICAL\fR mappings are not applicable, and
\fBNATURAL\fR mapping will apply to the piece of surface created by each
Bezier curve separately. The \fBNATURAL\fR mapping uses the curve parameter
u along each curve as \fIx\fR coordinate and goes from 0 to 1 around the
perimeter of the rotational surface on the other axis
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle is returned.
'@endhelp
'
'@help: sipp/primitves/BezierPatch
'@brief: Define an object using a Bezier patch.
.TP
.nf
\fBSippBezierPatch \fIresolution vertexlist patchlist shaderhandle\fR [\fItexture\fR]
.fi
Define an object using a Bezier patch.
\fIresolution\fR is the  number of polygons to tessellate the surface into.
\fIVertexlist\fR isA list of vertices of the curve.  Each element of the list
is a list contain the X, Y and Z vertices.
.sp
\fIPatchlist\fR is list of indices into \fIvertexlist\fR that defined the
patch.  Each element of the list contains a list 4 lists.  Each of these lists
contain 4 indices.  The indices are zero based.  For example:
.sp
.nf
.ft CW
    {{ 0  1  2  3} {35 36 37 38} {28 29 30 31} {21 22 23 24}}
    {{21 22 23 24} {14 15 16 17} { 7  8  9 10} { 0  1  2  3}}
    {{ 3  4  5  6} {38 39 40 41} {31 32 33 34} {24 25 26 27}}
    {{ 6  5  4  3} {13 12 11 10} {20 19 18 17} {27 26 25 24}}}
.ft R
.fi
.sp
\fIShaderhandle\fR is handle to a previously created surface shading
specification.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
The texture coordinates are a bit special for the Bezier patches.
\fBCYLINDRICAL\fR and \fBSPHERICAL\fR coordinates are not applicable, if they
are specified, SIPP will use \fBNATURAL\fR anyway. The \fBNATURAL\fR mapping is
a two dimensional mapping using the surface parameters u and v, see
figure below. Note that these parameters range from 0 to 1 within each
patch!
.sp
An object handle is returned.
'@endhelp
'
'@help: sipp/primitves/BezierFile
'@brief: Create an object for a Bezier curve or patch specified in a data file.
.TP
\fBSippBezierFile \fIfileid resolution shaderhandle\fR [\fItexture\fR]
.br
Read a file specified by \fIfileid\fR, containing descriptions of a set of
Bezier patches or Bezier curves. The surface is shaded with the algorithm and
surface parameters specified by \fIshaderhandle\fR.  See the \fBsipp
primitives\fR manual page for a description of this file format.
The \fB SippBezierCurve\fR and \fBSippBezierPatch\fR commands are the preferred
method of creating Bezier surfaces, this command is provided for compatibility
with existing SIPP data files.
.sp
If the file contains descriptions of patches, these patches will be tesselated
into \fIresolution\fR by \fIresolution\fR polygons and installed in a
\fBsipp\fR object as one surface.
.sp
If the file contains descriptions of curves, a rotational surface will be
created by rotating these curves about the y-axis. The resulting surface will
be tesselated into polygons and installed in a \fBsipp\fR object as one
surface.
.sp
\fITexture\fR is the texture mapping algorithm to use for the object.
The texture coordinates are a bit special for the Bezier patches.
\fBCYLINDRICAL\fR and \fBSPHERICAL\fR coordinates are not applicable, if they
are specified, SIPP will use \fBNATURAL\fR anyway. The \fBNATURAL\fR mapping is
a two dimensional mapping using the surface parameters u and v, see
figure below. Note that these parameters range from 0 to 1 within each
patch!
'@:.sp
'@:See the help file sipp/Texture-Map for more details.
.sp
An object handle that references the new object is returned.
'@endhelp
'
.SH "LIGHT SOURCE AND VIEWING COMMANDS"
These commands specify the light sources and the viewing parameters.
'
'@help: sipp/lights/LightSourceCreate
'@brief: Create a new lightsource in the scene.
.TP
\fBSippLightSourceCreate \fR{\fIx y z\fR} \fIcolor type\fR
.br
Create a new lightsource in the scene.
If a directional lightsource is created these \fR{\fIx\ y \z\fR} specifies a
vector in the direction that the light emanates
the lightsource.  If it is a point lightsource they specify
the exact location of it. \fIColor\fR specifies the color and intensity of 
the light.  \fIType\fR is either `\fBDIRECTION\fR' or `\fBPOINT\fR'.  A light
handle is returned by the command.
'@endhelp
'
'@help: sipp/lights/SpotLightCreate
'@brief: Create a new spotlight in the scene.
.TP
\fBSippSpotLightCreate \fIposition point opening color type shadow\fR
.br
Create a new spotlight in the scene.  \fIPosition\fR is the coordinates of the
spotlight.  \fIPoint\fR is a point at which the spotlight is pointing. It is
in the middle of the lightcone. 
\fIOpening\fR is the opening angle of the lightcone. The cone
defined will be completely lit, a soft edged lightcone will start to
blend out outside this angle.  If the number is
prefixed with an "R", then it is taken as radians, if it is prefixed with "D",
then it is taken as radians.  If not prefix, then degrees is assumed.
\fIColor\fR is The color of the emitted light.
\fIType\fR is the type of spotlight to create. Should be either "SHARP" or
"SOFT".  \fIShadow\fR is a boolean that indicates if light from the
spotlight will be able to cast shadows.  Whether shadows actually are cast or
not depend on which value set by the \fBSippShadows\fR command.  A light
handle is returned by the command.
'@endhelp
'
'@help: sipp/lights/LightDestruct
'@brief: Delete lightsources or spotlights.
.TP
\fBSippLightDestruct \fIlightlist\fR
.br
Delete lightsources or spotlights. \fILightlist\fR is a list of light handles
to destruct.
'@endhelp
'
'@help: sipp/lights/LightSourcePut
'@brief: Modify the direction or position of a lightsource.
.TP
\fBSippLightSourcePut \fIlight \fR{\fIx y z\fR}
.br
This command is used to modify the direction, or position, of a
lightsource. If the light is a directional light, then \fIx, y, z\fR
are a vector in the direction that the light emanates.
If the light is a point light source, then \fIx, y, z\fR are the coordinates
of the new location of a light.  
'@endhelp
'
'@help: sipp/lights/SpotLightPos
'@brief: Modify the position of a spotlight.
.TP
\fBSippSpotLightPos \fIlight \fR{\fIx y z\fR}
.br
Modify the position of a spotlight. \fIX, y, z\fR are the coordinates
of the new location of a light.  
'@endhelp
'
'@help: sipp/lights/SpotLightAt
'@brief: Modify the position a spotlight is pointing at.
.TP
\fBSippSpotLightAt \fIlight \fR{\fIx y z\fR}
.br
Modify the position a spotlight is pointing at. \fIX, y, z\fR are the
coordinates of the  point that the light should point at.
'@endhelp
'
'@help: sipp/lights/SpotLightOpening
'@brief: Modify the opening angle of the light cone of a spotlight.
.TP
\fBSippSpotLightOpening \fIlight opening\fR
.br
Modify the opening angle of the lightcone of a spotlight. 
\fIOpening\fR is the opening angle of the lightcone. The cone
defined will be completely lit, a soft edged lightcone will start to
blend out outside this angle.  If the number is
prefixed with an "R", then it is taken as radians, if it is prefixed with "D",
then it is taken as radians.  If not prefix, then degrees is assumed.
'@endhelp
'
'@help: sipp/lights/SpotLightShadows
'@brief: Turn shadow casting on or off for a spotlight. 
.TP
\fBSippSpotLightShadows \fIlight flag\fR
.br
Turn shadow casting on or off for a specific spotlight. 
\fIShadow\fR is a boolean that indicates if light from the
spotlight will be able to cast shadows.
'@endhelp
'
'@help: sipp/lights/LightColor
'@brief: Change the color of a lightsource or a spotlight.
.TP
\fBSippLightColor \fIlight color\fR
.br
Change the color of the emitted light from a lightsource or a spotlight.
fIColor\fR is the new color of the light.
'@endhelp
'
'@help: sipp/lights/LightActive
'@brief: Turn a lightsource or a spotlight on or off.
.TP
\fBSippLightActive\fI light flag\fR
.br
Turn a lightsource or a spotlight on or off. \fIFlag\fR is boolean indicating
if a light should be active.
'@endhelp
'
'@help: sipp/view/CameraCreate
'@brief: Create a new camera.
.TP
\fBSippCameraCreate \fR[\fIposition\fR] [\fIpoint\fR] [\fIupvector\fR] [\fIfocal\fR]
.br
Create a new camera. \fIPositon\fR is the point to world coordinates of the
camera. \fIPoint\fR is a world corrdinates point that he camera is to look at.
\fIUpvector\fR is a vector defining the up direction.  The only constraint on
this vector is that it must not be parallel to the viewing direction.
.sp
\fIFocal\fR the focal ratio of the "camera". This is the ratio between the
distance from the viewpoint to the screen and half the screen height.  A large
focal factor will result in a wide angle view while a small factor will give a
telescopic effect.
.sp
.nf
.ft CW
                                screen
                                |
                                | s
    viewpoint                   |
        *-----------------------|
                    d           |
                                |
                                |

        focal = s / d
.ft R
.fi
.sp
If any of the parameters is omitted or has a empty value, then that parameter
is defaulted.  A camera handle is returned.  If a camera is not created, the
default camera, \fBSTDCAMERA\fR, is used.
'@endhelp
'
'@help: sipp/view/CameraDestruct
'@brief: Delete cameras.
.TP
\fBSippCameraDestruct \fIcameralist\fR
Delete one or more cameras.  \fICameralist\fR is a list of camera handles,
'@endhelp
'
'@help: sipp/view/CameraParams
'@brief: Alter the parameters of a camera.
.TP
.nf
\fBSippCameraParams \fIcamera\fR \fR[\fIposition\fR] [\fIpoint\fR] [\fIupvector\fR] [\fIfocal\fR]
.fi
.br
Alter the parameters of a camera.  \fICamera\fR is the camera handle, all
other arguments are the same as for \fBSippCameraCreate\fR. If any of the
parameters is omitted or has a empty value, then that parameter is unchanged.
'@endhelp
'
'@help: sipp/view/CameraUse
'@brief: Specify the camera to use as the current viewpoint.
.TP
\fBSippCameraUse \fIcamera\fR
Specify the camera to use as the current viewpoint.
'@endhelp
'
.SH "PORTABLE BITMAP RENDERING COMMANDS"
.PP
These commands are used for rendering to files defined by the PBMPLUS toolkit,
either PPM files for fully rendered images or PBM files for line images.
'
'@help: sipp/ppm/PPMRender
'@brief: Render a scene to a PPM file.
.TP
\fBSippPPMRender [\fI-flag\fR] \fIfileid xsize ysize\fR [\fImode\fR]
    [\fIoversample\fR]
.br
This command does the rendering of a scene. The image is created with the
pixel dimensions \fIxsize\fR by \fIysize\fR.  \fIFileid\fR is an id of a open
file, as returned by the \fBopen\fR command.  The image is written in the
Portable Pixel Map (PPM) format unless \fImode\fR is \fBLINE\fR, in which case
it is written in the Portable Bit Map (PBM) format.  fIMode\fR decides in
which mode the image should be rendered and should be one of \fBPHONG\fR,
\fBGOURAUD\fR, \fBFLAT\fR or \fBLINE\fR.  The integer value \fIoversampling\fR
controls how much oversampling should be done for anti-aliasing. Each pixel
will be the average of a \fIoversampling\fR X \fIoversampling\fR matrix of
subpixels.  \fIOversampling\fR is ignored if mode is \fBLINE\fR.
.sp
The optional flag \fI-flag\fR is used to do interlaced rendering, which is
useful for animations.  If \fB-ODD\fR is specified, the odd scanlines are
rendered, if \fB-EVEN\fR is specified, then even scanlines are rendered.
If \fB-BOTH\fR is specified or the flag is omitted, then all scanlines
are rendered.  Interlaced rendering is not allowed if \fImode\fR is \fBLINE\fR.
'@endhelp
'
.SH "RLE FILE MANIPULATION AND RENDERING COMMANDS"
.PP
These commands are used to open, manipulate and render to RLE files as defined
by the Utah Raster Toolkit.
'
'@help: sipp/rle/RLEOpen
'@brief: Open or create a RLE file.
.TP
\fBSippRLEOpen \fIfilename mode\fR
.br
Open the RLE file \fIfilename\fR.  \fIMode\fR is "\fBw\fR" to create or
truncate the file and "\fBa\fR" to create or open for appending. The append
option allows for creating RLE files containing several images.
The command returns a handle to the RLE file.
\fIFilename\fR follows the full semantics of the file name argument to the
Tcl \fBopen\fR command.  Thus the RLE file may be sent to a pipeline, which
could be a viewer or post-processing program.  Thus to render an image directly
into the RLE X11 file view:
.sp
.nf
.ft CW
    set image [SippRLEOpen "|getx11" "w"]
.ft R
.fi
.sp
'@endhelp
'
'@help: sipp/rle/RLEClose
'@brief: Close a RLE file.
.TP
\fBSippRLEClose \fIrlehandle\fR
.br
Close the RLE file specified by \fIrlehandle\fR.
'@endhelp
'@help: sipp/rle/RLEPutCom
'@brief: Set a comment in a RLE file.
.TP
\fBSippRLEPutCom \fIrlehandle name\fR [\fIvalue\fR]
.br
Set a comment in the RLE file specified by \fIrlehandle\fR.  A RLE comment
is similar to an environment variable.  This sets a comment of name \fIname\fR
to the string \fIvalue\fR.  If \fIvalue\fR is omitted, then the name is saved,
without a value.
'@endhelp
'
'@help: sipp/rle/RLEGetCom
'@brief: Return the value of comment from a RLE file.
.TP
\fBSippRLEGetCom rlehandle name [retvar | {}]
.br
Return the value of comment identified by \fIname\fR from a RLE file specified
by \fIrlehandle\fR.  If \fIretvar\fR is specified the command returns
`\fB1\fR' if the comment is found and `\fB0\fR' if it is not found.  The value
of the comment is returned in the variable \fIretvar\fR, unless \fIretvar\fR
is `{}', in which the only the presence of the variable is returned, but not
value.  If \fIretvar\fR is not specified, the command returns the value of the
comment or an empty string if the comment has no value or is not found.
'@endhelp
'
'@help: sipp/rle/RLEDelCom
'@brief: Delete a comment from a RLE file.
.TP
\fBSippRLEDelCom \fIrlehandle name\fR
.br
Delete the comment \fIname\fR from the RLE file specified by \fIrlehandle\fR.
'@endhelp
'
'@help: sipp/rle/RLERender
'@brief: Render a scene to a RLE file.
.TP
\fBSippRLERender\fR [\fI-flag\fR] \fIrlehandle xsize ysize\fR [\fImode\fR]
    [\fIoversample\fR]
.br
This command does the rendering of a scene. The image is created with the
pixel dimensions \fIxsize\fR by \fIysize\fR.  It is written the RLE file
specified by \fIrlehandle\fR.  \fIMode\fR decides in
which mode the image should be rendered and should be one of \fBPHONG\fR,
\fBGOURAUD\fR, \fBFLAT\fR, or \fBLINE\fR.  The integer value
\fIoversampling\fR controls how much oversampling should be done for
anti-aliasing. Each pixel will be the average of a \fIoversampling\fR X
\fIoversampling\fR matrix of subpixels.  \fIOversampling\fR is ignored for
\fBLINE\fR mode.
.sp
The optional flag \fI-flag\fR is used to do interlaced rendering, which is
useful for animations.  If \fB-ODD\fR is specified, the odd scanlines are
rendered, if \fB-EVEN\fR is specified, then even scanlines are rendered.
If \fB-BOTH\fR is specified or the flag is omitted, then all scanlines
are rendered.  Interlaced rendering is not allowed if \fImode\fR is \fBLINE\fR.
'
'@endhelp
'
.SH "MISCELLANEOUS COMMANDS"
.PP
A few more useful commands.
'
'@help: sipp/misc/ShowBackFaces
'@brief:  Enable or disable backface culling or hidden line elimination.
.TP
\fBSippShowBackFaces \fR[\fIflago\fR]
.br
This function can be used if backface culling should not be performed. If the
boolean \fIflag\fR is true backfacing polygons are "inverted" (their surface
normal is negated) and then rendered. This is useful if an object description
does not use a consequent ordering of the vertices.  This also controls hidden
line elimination when doing line rendering.  The rendering step can be slowed
down considerably since, on the average, twice as many polygons need to be
rendered. The default is that backface culling should be performed (false).
If \fIflag\fR is not specified, the current value is returned.
'@endhelp
'
'@help: sipp/misc/Background
'@brief: Set the background color of the image.
.TP
\fBSippBackground \fR[\fIcolor\fR]
.br
Set the background color of the image to \fIcolor\fR.  If \fIcolor\fR is not
specified, the the current background color is returned.
'@endhelp
'
'@help: sipp/misc/LineColor
'@brief: Set the color for lines in RLE line mode rendering.
.TP
\fBSippLineColor \fR[\fIcolor\fR]
.br
Set the color of lines to use in line mode rendering to \fIcolor\fR.  This
is only useful when rendering to RLE files, it is ignored for PBM files.
If \fIcolor\fR is not specified, the the current line color is returned.
'@endhelp
'
'@help: sipp/misc/ShowShadows
'@brief:  Globally enable or disable shadows.
.TP
\fBSippShowShadows \fIflag\fR [\fIsize\fR]
.br
This command globaly enables or disables the casting of shadows by objects.
If the boolean \fIflag\fR is true, then shodows will be cast by lights that
have shadows casting enabled.  \fISize\fR defines the size of the depth map
used to render the shadows.  As a rule of thumb one could say that the depth
maps should be at least as large as the image itself but this may vary from
case to case.  \fISize\fR is omitted when \fIflag\fR is false.
'@endhelp
'
'@help: sipp/misc/Info
'@brief:  Return various pieces of info about Tcl-Sipp.
.TP
\fBSippInfo \fIattribute\fR
.br
This command returns various pieces of information about Tcl-Sipp.  The
following values may be specified for \fIattribute\fR:
.IP
o \fBversion\fR - Returns the full Tcl-SIPP version.  This includes the SIPP
library version number, a suffix indicating the Tcl-SIPP interface version and
the Tcl-SIPP interface patch level (if its been patched).
.IP
o \fBsippversion\fR - Returns the SIPP library version number that Tcl-SIPP is
linked with.
.IP
o \fBtsippversion\fR - Returns the Tcl-SIPP interface version (the suffix in
the full version).
.IP
o \fBtsipppatchlevel\fR - Returns the Tcl-SIPP interface patchlevel.
.IP
o \fBrle\fR - Returns 1 if Tcl-SIPP is linked with the RLE libraries and can
product RLE files.  Returns 0 if RLE file support is not available.
'@endhelp
'
'@help: sipp/misc/MkVector
'@brief: Create a vector between two points.
.TP
\fBSippMkVector \fR{\fIpt0_x pt0_y pt0_z\fR} {\fIpt1_x pt1_y pt1_z\fR}
.br
Create a vector in the director going from the point defined by \fIpt0_x,
pt0_y, pt0_z\fR to the point defined by\fIpt1_x, pt1_y, pt1_z\fR.
'@endhelp
'
'@help: sipp/misc/MatMult
'@brief: Multiply two matrices.
.TP
\fBSippMatMult \fImatrix1 matrix2\fR
.br
Multipy \fImatrix1\fR by \fImatrix2\fR.
'@endhelp
'
.SH "NOTES"
.IP
o Extended Tcl handles signals that are trapped by setting a flag indicating
that a particular signal occured and then processing it when the current
Tcl command has completed.  The Tcl-SIPP rendering commands may take a very
long time.  Extended Tcl catches SIGINT by default.  If you wish to be able
to always abort your scripts with SIGINT, issue the following command from
your script:
.br
.nf
.ft CW
        signal default SIGINT
.ft R
.fi
.IP
o Although the Tcl interface tries to validate argument data, SIPP itself is
not very bullet proof.  Incorrect or out of range data can cause unfriendly
failures, including core dumps.  If you have problems, check the values being
passed to a command.
.IP
o Memory associated with shaders is never released, as SIPP does not keep a
use count on the Surf_desc data structure once associated with a surface.
However, the memory associated with shaders is small.
.IP
o No command is implement using the mask shader.
.IP
o While texture coordinates may be specified for a surface, there is currently
no way to use them.
.IP
o The Bezier File command returns errors to stderr rather than via the Tcl
error mechanism.
'
.SH "AUTHOR OF THE TCL INTERFACE"
Mark Diekhans (markd@grizzly.com).  Please e-mail questions, comments
and suggestions.
